/*
 * Rubik's cube interface
 *
 * Copyright (C) 2007, 2008 Emmanuel Thome'
 * <Emmanuel dot Thome at normalesup dot org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef PERM_HPP_
#define PERM_HPP_

#include <vector>
#include <istream>
#include <ostream>
#include <sstream>
#include <string>
#include <cstdlib>

template<int n>
class perm {
	int	v[n];
public:
	int& operator[](int i) { return v[i]; }
	const int& operator[](int i) const { return v[i]; }
	perm inverse() const {
		perm s;
		for(int i=0;i<n;i++) {
			s[v[i]]=i;
		}
		return s;
	}
	bool is_perm() const {
		std::vector<bool> t(n,false);
		for(int i=0;i<n;i++) {
			if (t[v[i]]) return false;
			t[v[i]]=true;
		}
		return true;
	}
	bool operator==(const perm<n>& o) const {
		for(int i=0;i<n;i++) {
			if (v[i]!=o[i]) return false;
		}
		return true;
	}
	perm operator*(const perm& s) const {
		perm r;
		for(int i=0;i<n;i++) {
			r[i]=v[s[i]];
		}
		return r;
	}
	perm& operator*=(const perm& s) {
		*this = (*this) * s;
		return *this;
	}
public:
	/* XXX XXX change to private (exposed for teting only) */
	void set_tab(const int c[n]) {
		for(int i=0;i<n;i++) v[i]=c[i];
	}

	void set_tab(std::vector<int> c) {
		ASSERT_ALWAYS(c.size()==n);
		for(int i=0;i<n;i++) v[i]=c[i];
		ASSERT(is_perm());
	}
private:
	void set_id() { for(int i=0;i<n;i++) v[i]=i; }
        static int pgcd(int a, int b) {
		for(;b!=0;) { int r=a%b; a=b; b=r; } return a;
	}
        static int ppcm(int a, int b) {
                int d=pgcd(a,b); return (a/d)*(b/d)*d;
	}
public:
	std::vector<std::vector<int> > get_cycles() const {
		std::vector<std::vector<int> > ccs;
		std::vector<bool> seen(n,false);
		for(int i=0;i<n;i++) {
			if (seen[i] || v[i]==i) continue;
			std::vector<int> c;
			int j=i;
			do {
				c.push_back(j);
				seen[j]=true;
				j=v[j];
			} while (j!=i);
			ccs.push_back(c);
		}
		return ccs;
	}
	int order() const {
		std::vector<std::vector<int> > ccs=get_cycles();
		int v=1;
		for(unsigned int i=0;i<ccs.size();i++) v=ppcm(v,ccs[i].size());
		return v;
	}
	void set_from_cycles(std::vector<std::vector<int> > cc) {
		set_id();
		for(unsigned int i=0;i<cc.size();i++) {
			std::vector<int> c = cc[i];
			unsigned int l = c.size();
			for(unsigned int j=0;j<l;j++) {
				v[c[j]] = c[(j+1) % l];
			}
		}
		ASSERT(is_perm());
	}
	class perm_parser : public parser_base {
		bool parse_cycle(std::vector<int>& c) {
			if (!ptry('(')) return false;
			c.clear();
			for(;numlen();) {
				c.push_back(want_ui());
			}
			want(')');
			return true;
		}
		bool parse_cycle_dec(perm& r) {
			std::vector<std::vector<int> >cc;
			std::vector<int> c;
			if (!parse_cycle(c)) return false;
			cc.clear();
			cc.push_back(c);
			for(;ptry('.');) {
				require(parse_cycle(c));
				cc.push_back(c);
			}
			require(at_end());
			r.set_from_cycles(cc);
			return true;
		}
		bool parse_immediate(perm& r) {
			if (!ptry('{')) return false;
			std::vector<int> v;
			for(;numlen();) {
				v.push_back(want_ui());
			}
			want('}');
			r.set_tab(v);
			return true;
		}
		bool parse_id(perm& r) {
			if (!(ptry("id") || ptry("e") || ptry("1")))
				return false;
			r.set_id();
			return true;
		}
		public:
        	perm_parser(const char * obuf, unsigned int osize) :
			parser_base(obuf,osize) {}
		bool do_parse(perm& res) {
			return	parse_cycle_dec(res) ||
				parse_immediate(res) ||
				parse_id(res);
		}
	};
public:
	std::istream& operator>>(std::istream& is) {
		const unsigned int buffer_size=parser_base::buffer_size;
		char    buffer[buffer_size];
		is.getline(buffer,buffer_size);
		if (is.gcount()) {
			perm_parser j(buffer,is.gcount());
			if (!j.do_parse(*this)) {
				is.setstate(std::ios::failbit);
			}
		}
		return is;

	}
	std::ostream& operator<<(std::ostream& os) const {
		os << "{";
		for(int i=0;i<n;i++) {
			os << " " << v[i];
		}
		os << " }";
		return os;
	}

	unsigned int inv_seg() const {
		unsigned int r;
		for(r=0;r < n && v[r]==r;r++);
		return r;
	}
	perm() { set_id(); }
	perm(const perm<n>& t) { set_tab(t.v); }
};

template<int n>
std::istream& operator>>(std::istream& is, perm<n> & a)
{
        return a.operator>>(is);
}

template<int n>
std::ostream& operator<<(std::ostream& os, const perm<n> & a)
{
        return a.operator<<(os);
}


#endif	/* PERM_HPP_ */
