/*
 * Rubik's cube interface
 *
 * Copyright (C) 2007, 2008 Emmanuel Thome'
 * <Emmanuel dot Thome at normalesup dot org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <iostream>
#include <vector>
#include <list>
#include <queue>
#include <iterator>
#include <cstring>
#include <fstream>

#include "rubik.hpp"

sift_table<48> rubik_sift;

static const char * base_change_string =
	"{ 30 4 27 6 2 33 0 24 32 7 34 35 1 25 26 3 28 13 14 15 8 9 10 43 23 47 46 17 38 37 19 41 45 16 36 22 18 42 20 39 44 21 40 12 11 31 5 29 }";

static perm<48> base_change;
static perm<48> base_change_inv;

static const char * permutations_faces[] = {
	// top
	"(0 2 7 5).(1 4 6 3).(8 47 14 11).(9 46 15 12).(10 45 16 13)",
	// close
	"(5 14 34 25).(6 21 33 18).(7 29 32 10).(11 13 28 26).(12 20 27 19)",
	// right
	"(2 42 34 13).(4 44 36 20).(7 47 39 28).(14 16 31 29).(15 22 30 21)",
	// far
	"(0 23 39 16).(1 17 38 22).(2 8 37 31).(40 42 47 45).(41 44 46 43)",
	// left
	"(0 11 32 40).(3 19 35 43).(5 26 37 45).(8 10 25 23).(9 18 24 17)",
	// bottom
	"(23 26 29 42).(24 27 30 41).(25 28 31 40).(32 34 39 37).(33 36 38 35)",
};
#define	SOURCE	permutations_faces
#define	NGENS	(sizeof(SOURCE)/sizeof(SOURCE[0]))

const int map_48_54[48] = {
	0,1,2,3,5,6,7,8,
	36,37,38,9,10,11,18,19,20,
	39,41,12,14,21,23,
	42,43,44,15,16,17,24,25,26,
	45,46,47,48,50,51,52,53,
	35,34,33,
	32,30,
	29,28,27};

std::vector<int> corresp_48_to_54(const perm<48> & tl)
{
	perm<48> t = base_change_inv * tl * base_change;
	std::vector<int> res(54,-1);
	for(int i=0;i<54;i++) res[i]=i;
	for(int i=0;i<48;i++) {
		res[map_48_54[i]]=map_48_54[t[i]];
	}
	return res;
}

perm<48> corresp_54_to_48(const std::vector<int>& r)
{
	perm<48> t;
	std::vector<int> invmap(54,-1);
	for(int i=0;i<48;i++) { invmap[map_48_54[i]]=i; }
	for(int i=0;i<48;i++) {
		t[i] = invmap[r[map_48_54[i]]];
	}
	return base_change * t * base_change_inv;
}

int main(int argc, char **argv)
{
	rubik_sift.quiet();

	{
		std::istringstream is(base_change_string);
		is >> base_change;
		base_change_inv=base_change.inverse();
	}

	if (argc>1 && strcmp(argv[1],"--do-sift")==0) {
		argc--;
		argv++;

		perm<48> faces[NGENS];

		for(unsigned int i=0;i<NGENS;i++) {
			std::istringstream is(SOURCE[i]);
			is >> faces[i];
			// std::cout << faces[i] << "\n";
			faces[i] = base_change * faces[i] * base_change_inv;
			rubik_sift.generator(faces[i]);
		}
		rubik_sift.fix_generators();

		int pass;
		for(pass=0;!rubik_sift.finished();pass++) {
			std::cout << "Sifting, pass " << pass << "\n";
			rubik_sift.apply();
		}
		std::cout << "Finished sifting in "<< pass <<" passes\n";
		std::cout << "Group order: " << rubik_sift.group_order() << "\n";

		rubik_sift.profile(std::cout);

		std::ofstream data_file;
		data_file.open("rubik.sft", std::ios_base::out);
		data_file << rubik_sift ;
		data_file.close();
	} else {
		std::ifstream data_file;
		data_file.open("rubik.sft", std::ios_base::in);
		if (!data_file.is_open()) {
			std::cerr << "Cannot read file !\n";
			return 255;
		}
		try {
			data_file >> rubik_sift;
		} catch (parse_error x) {
			std::cerr << "Parse error reading file : "
				<< x.msg << "\n";
		}
	}

	start_interface(argc,argv);
}
